#include "itkLinear2DIndexInterpolator.h"
#include <cstdlib>

namespace itk
{

Linear2DIndexInterpolator::
Linear2DIndexInterpolator(int x1, int y1, int x2, int y2) :
  m_x1(x1), m_y1(y1), m_x2(x2), m_y2(y2)
{
  // Initialize bresenham-like interpolation of the y coordinate.
  // The differences to an actual bresenham-line are
  // 1) Iteration is always over x, even if y2-y1 > x2-x1
  // 2) The y coordinate is always rounded downwards, not to the closest
  //    integer.
  int dx = m_x2 - m_x1;
  int steps = std::abs(dx);
  int dy = m_y2 - m_y1;

  if (dx < 0)
  {
    m_xIncrement = -1;
  }
  else if (dx == 0)
  {
    m_xIncrement = 0;
  }
  else
  {
    m_xIncrement = 1;
  }
 
  // We need m_yIncrement = floor(dy/steps). The following is an integer only way to
  // say that.
  if (steps == 0)
  {
    m_yIncrement = 0;
  }
  else
  {
    m_yIncrement = dy / steps;
    if (dy < 0 && dy % steps != 0)
      --m_yIncrement;
  }

  // In each step, we should increment y by dy/steps, but only increment by
  // floor(dy/steps), leaving an error of 0 <= dy/steps - floor(dy/steps) < 1.
  // This error is accumulated. The integer part of the accumulation overflows
  // into y.
  // To avoid floating point arithmetic, everthing is scaled by the factor
  // steps.
  m_yIncrementError = dy - m_yIncrement * steps;
  m_yMaximalError = steps;
  m_yReduceErrorAfterOverflow = steps;

  GoToBegin();
}

void
Linear2DIndexInterpolator::
GoToBegin()
{
  m_x = m_x1;
  m_y = m_y1;
  m_yAccumulateError = 0;
  m_StepsLeft = std::abs(m_x2 - m_x1);
}

void
Linear2DIndexInterpolator::
operator++()
{
  // Take a unit step in x direction, update y to floor(y') for the floating
  // point y' corresponding to the new x on the line.
  m_x += m_xIncrement;
  m_y += m_yIncrement;
  m_yAccumulateError += m_yIncrementError;
  if (m_yAccumulateError >= m_yMaximalError) {
    ++m_y;
    m_yAccumulateError -= m_yReduceErrorAfterOverflow;
  }
  --m_StepsLeft;
}

int
Linear2DIndexInterpolator::
GetX() const
{
  return m_x;
}
  
int
Linear2DIndexInterpolator::
GetY() const
{
  return m_y;
}

bool
Linear2DIndexInterpolator::
IsYExact() const
{
  return !m_yAccumulateError;
}

bool
Linear2DIndexInterpolator::
IsAtEnd() const
{
  return m_StepsLeft < 0;
}

bool
Linear2DIndexInterpolator::
IsAtLast() const
{
  return m_StepsLeft == 0;
}

float
Linear2DIndexInterpolator::
YError() const
{
  // Return the offset from the current integer y to the exact floating point
  // y. This is non-negative, because y is rounded downwards.
  return m_yMaximalError == 0 ? 0 : static_cast<float>(m_yAccumulateError) / static_cast<float>(m_yMaximalError);
}

} // end namespace itk
