#ifndef __itkLinear2DIndexInterpolator_h
#define __itkLinear2DIndexInterpolator_h

#include "itkMacro.h"


namespace itk
{
// Forward declaration of a friend class
class Linear2DIndexComparator;

/**
 * \class Linear2DIndexInterpolator
 * \brief Interpolator that iterates one value and gives an integer
 * approximization of the linear interpolation of a second.
 *
 * Given two coordinates (x1, y1), (x2, y2), the interpolator interates
 * over the range of x with step size 1 (or -1 if x2 < x1). After i of n
 * steps, it approximates y to floor(y1 + i/n(y2-y1)).
 *
 * The main purpose of this class is the use in polygon scan conversion.
 *
 * \sa PolygonScanConversionIterator, LineIterator
 */
class Linear2DIndexInterpolator
{
public:
  /** Standard class typedefs. */
  typedef Linear2DIndexInterpolator Self;

  /** Run-time type information (and related methods). */
  itkTypeMacro(Linear2DIndexInterpolator, None);

  /** Constructor establishes an interpolator to walk from
   * x1 to x2, interpolating y from y1 to y2 */
  Linear2DIndexInterpolator(int x1, int y1, int x2, int y2);

  /** Move to the start index */
  void GoToBegin();

  /** Move current x towards x2 by 1 and update y accordingly. */
  void operator++(); 

  /** Get the current x */
  int GetX() const;

  /** Get the current y */
  int GetY() const;

  /** Test whether the current y is different from the exact linear
   * interpolation or not  */
  bool IsYExact() const;

  /** Have all x been iterated? */
  bool IsAtEnd() const;

  /** Is the current x at x2? */
  bool IsAtLast() const;

protected:
  /** Start and end point. */
  int m_x1;
  int m_y1;
  int m_x2;
  int m_y2;
  
  /** Integer increment for each step. x is incremented by 1 for x1 < x2 and
   * decremented otherwise */
  int m_xIncrement;
  int m_yIncrement;

  /** Floating point arithmetic is avoided by using a bresenham-like approach. */
  int m_yAccumulateError;
  int m_yIncrementError;
  int m_yMaximalError;
  int m_yReduceErrorAfterOverflow;

  /** Current position. */
  int m_x;
  int m_y;
  int m_StepsLeft;

private:
  /** The exact linear interpolation of y at the current position is
   * GetY()+YError() with 0 <= YError() < 1.
   * This method makes sorting by exact y position possible. */
  float YError() const;

  friend class Linear2DIndexComparator;
};

} // end namespace itk
#endif
