#ifndef __itkPolygonScanConversionConstIterator_h
#define __itkPolygonScanConversionConstIterator_h

#include "itkLinear2DIndexInterpolator.h"
#include "itkLinear2DIndexComparator.h"
#include <vector>
#include <set>

namespace itk
{

/**
 * \class PolygonScanConversionConstIterator
 * \brief Iterator that visits all points inside a complex polygon.
 *
 * PolygonScanConversionConstIterator is an iterator that visits all points inside
 * or on the border of a complex polygon.
 * The polygon has to be on a 2D plane orthogonal to any axis of an ND image.
 *
 * The implementation follows the active edge table approach with the
 * odd/even inclusion rule, beautifully described in
 * The Simplicity of Complex Polygons, Michael Abrash, Dr. Dobb's Journal
 * June 1991, v16, n6, p139(6)
 *
 * It's possible that the set of iterated points is not connected, for
 * example the triangle (0,0), (0,1), (3,2) looks like this:
 *  #
 *  ##
 *     #
 * This class was contributed by Benjamin King, Experimentelle
 * Radiologie, Medizinische Hochschule Hannover.
 *
 */
template<class TImage>
class PolygonScanConversionConstIterator
{
public:
  /** Standard class typedefs. */
  typedef PolygonScanConversionConstIterator Self;

  /** Dimension of the image the iterator walks.  This constant is needed so 
   * that functions that are templated over image iterator type (as opposed to
   * being templated over pixel type and dimension) can have compile time
   * access to the dimension of the image that the iterator walks. */
  itkStaticConstMacro(ImageIteratorDimension, unsigned int,
                      TImage::ImageDimension);

  /** Index typedef support. */
  typedef typename TImage::IndexType          IndexType;
  typedef typename TImage::IndexValueType     IndexValueType;
  typedef std::vector<IndexType>              IndexListType;

  /** Offset typedef support. */
  typedef typename TImage::OffsetType         OffsetType;
  typedef typename TImage::OffsetValueType    OffsetValueType;

  /** Size typedef support. */
  typedef typename TImage::SizeType           SizeType;
  typedef typename TImage::SizeValueType      SizeValueType;

  /** Region typedef support */
  typedef typename TImage::RegionType         RegionType;

  /** Spacing typedef support */
  typedef typename TImage::SpacingType        SpacingType;

  /** Origin typedef support */
  typedef typename TImage::PointType          PointType;
  
  /** Image typedef support. */
  typedef TImage                              ImageType;

  /** PixelContainer typedef support. Used to refer to the container for the
   * pixel data. */
  typedef typename TImage::PixelContainer     PixelContainer;
  typedef typename PixelContainer::Pointer    PixelContainerPointer;
  
  /** Internal Pixel Type */
  typedef typename TImage::InternalPixelType  InternalPixelType;

  /** External Pixel Type */
  typedef typename TImage::PixelType          PixelType;

  /** Accessor type that convert data between internal and external
   * representations. */
  typedef typename TImage::AccessorType       AccessorType;

  /** Run-time type information (and related methods). */
  itkTypeMacro(LineConstIterator, None);

  /** Get the dimension (size) of the index. */
  static unsigned int GetImageIteratorDimension() 
    {
    return TImage::ImageDimension;
    }

  /** Get the pixel value */
  const PixelType & Get(void) const
    {
    return m_Image->GetPixel(m_CurrentImageIndex );
    }

private:
  /** To implement the active edge table algorithm the list of edges must be
   * sorted. std::multiset sorts automatically. */
  typedef std::multiset<itk::Linear2DIndexInterpolator*, itk::Linear2DIndexComparator> EdgeListType;

public:
  /** Constructor
   *
   * The vertices v1, ..., vn must be on a 2D plane orthogonal to an axis of
   * the coordinate system, i.e. only two components may be non-constant.  A
   * list of edges is generated by pairing adjacent vertices.  (vn, v1) is an
   * edge, too. 
   *
   * The first of the non-constant components is used to iterate over all scanlines,
   * the second is used to iterate over the points in one scanline. */
  PolygonScanConversionConstIterator(const ImageType *, const IndexListType &);

  /** Default destructor. */
  virtual ~PolygonScanConversionConstIterator();


  /** Set the scan conversion iterator to the first pixel
   *
   * Moves all lines from the active and finished egde lists to the global
   * edge list. At the same time, resets the line iterators. */
  void GoToBegin();

  /** Try to proceed in the current scanline. If there are no more pixels in
   * this line, try to proceed to the next scan line. If there are no more
   * scan lines, we are done. */
  void operator++();

  /** Get the index. This provides a read only reference to the index. */
  const IndexType & GetIndex() const
    {
    return m_CurrentImageIndex;
    }

  /** Past the end test. */
  bool IsAtEnd() const;

protected:
  /** Smart pointer to the source image. */
  typename ImageType::ConstWeakPointer m_Image;

  /** The three lists that drive the active edge table algorithm.
   * During the iteration
   * - m_globalEdgeList contains all polygon edges that did not cross a scan
   *   line yet
   * - m_activeEdgeList contains all polygon edges that are crossed by the
   *   current scan line
   * - m_finishedEdgeList contains all polygon edges that are already
   *   traversed.
   *
   * An edge is actually a Linear2DIndexInterpolator and manages it's
   * traversal itself. */
  EdgeListType m_globalEdgeList;
  EdgeListType m_activeEdgeList;
  EdgeListType m_finishedEdgeList;

  /** The current iterator position. */
  IndexType m_CurrentImageIndex;

  /** Direction to go from one scanline to the next. */
  int m_FirstDirection;

  /** Direction to go from one pixel to the next within a scanline. */
  int m_SecondDirection;

  /** Implementation detail: A complex polygon can have holes and thus a
   * scanline can have multiple segments of pixels inside the polygon.
   * m_currentEdge denotes the start of the active segment. */
  EdgeListType::const_iterator m_currentEdge;

  /** Past the end condition. */
  bool m_finished;

private:
  /** Copy construction and assignment are not implemented. */
  PolygonScanConversionConstIterator(const PolygonScanConversionConstIterator&);
  PolygonScanConversionConstIterator& operator=(const PolygonScanConversionConstIterator&);

  /** How many pixels are there from pos to the next edge? */
  int NumberOfPixelsToEndOfSegment(EdgeListType::const_iterator startEdge, int pos);

  /** Position the iterator at the beginning of the scan line
   * segment at or below the given active edge that contains at least one
   * pixel.
   *
   * oddActiveEdge must be just that. The list of active edges always contains
   * an even number of edges. The odd egdes start a scan line segment, the
   * even edges end it. Because of the inclusion strategy, a segment may
   * contain no pixel inside the polygon.
   *
   * If true is returned, the iterator is at it's new position.
   * If false is returned, there is no segment to go to in the current scan
   * line and the iterator is left where it was. */
  bool GoToFirstNonEmptyScanLineSegmentAtOrAfter(EdgeListType::const_iterator oddActiveEdge);

  /** Position the iterator at the beginning of the current scan line. Returns
   * false and leaves the iterator where it is if there is no pixel inside the
   * polygon in the current scan line. */
  bool GoToFirstScanLineSegment();

  /** Position the iterator at the beginning of the next segment in the
   * current scan line. Returns false and leaves the iterator where it is if
   * there is no next segment that contains pixels inside the polygon in the
   * current scan line. */
  bool GoToNextScanLineSegment();

  /** Position the iterator at the beginning of the next scan line. Returns
   * false and leaves the iterator where it is if there is no scan line left
   * that contains pixels inside the polygon. */
  bool GoToNextScanLine();

  /** Update the active edge list for traversal of the next scan line. */
  bool AdvanceActiveEdges();
};

} // end namespace itk

#ifndef ITK_MANUAL_INSTANTIATION
#include "itkPolygonScanConversionConstIterator.txx"
#endif

#endif
