#ifndef _itkPolygonScanConversionConstIterator_txx
#define _itkPolygonScanConversionConstIterator_txx

#include "itkPolygonScanConversionConstIterator.h"

namespace itk
{


template<class TImage>
PolygonScanConversionConstIterator<TImage>
::PolygonScanConversionConstIterator(const ImageType *image, const IndexListType &vertices)
{
  m_Image = image;

  if (vertices.size() != 0)
  {
    // Test if the vertices are on a 2D plane
    // 1) Find non-constant components
    std::vector<bool> constant(true, ImageIteratorDimension);
    IndexType firstIndex = *(vertices.begin());
    for (typename IndexListType::const_iterator it = vertices.begin(); it != vertices.end(); ++it)
    {
      for (int i=0; i < ImageIteratorDimension; ++i)
      {
        if ((*it)[i] != firstIndex[i])
        {
          constant[i] = false;
        }
      }
    }

    // 2) Count the number of components that change
    int count = 0;
    for (int i=0; i < ImageIteratorDimension; ++i)
    {
      if (!constant[i])
      {
        ++count;
      }
    }

    // 3) It must be exactly 2 components that change.
    //
    // If there are less, that means that there are no points inside or an odd
    // number of times on the border of the polygon. This is valid usage of
    // the class, however, and we only avoid to create any edges.
    //
    // If there are more components that change, the polygon can't be scan
    // converted by this class and we throw an exception.
    if (count < 2)
    {
      // Do nothing, polygon is empty
    }
    else if (count > 2)
    {
      // Not on a plane
      itkExceptionMacro("Vertices are not on a 2D plane orthogonal to an axis \
          of the coordinate system");
    }
    else
    {
      // In GoToBegin() the m_CurrentImageIndex will be moved to the first
      // index in scan conversion order.
      // Right now, m_CurrentImageIndex is set to any of the vertices to
      // initialize the constant components.
      m_CurrentImageIndex = *(vertices.begin());

      // Set first and second direction
      int i = 0;
      while (constant[i]) ++i;
      m_FirstDirection = i;

      ++i;
      while (constant[i]) ++i;
      m_SecondDirection = i;

      // Construct a list of all edges
      //
      // The polygon is mapped to the XY plane. In operator++(), the 2D
      // indices are mapped back to the original orientation.
      for (typename IndexListType::const_iterator it = vertices.begin(); it != vertices.end(); ++it)
      {
        const IndexType &v1 = *it;
        const IndexType &v2 = (it+1 == vertices.end() ? *(vertices.begin()) : *(it+1));

        IndexValueType v1x = v1[m_FirstDirection];
        IndexValueType v1y = v1[m_SecondDirection];
        IndexValueType v2x = v2[m_FirstDirection];
        IndexValueType v2y = v2[m_SecondDirection];

        // Skip vertical lines
        if (v1x == v2x)
        {
          continue;
        }

        // Scan from low x to high x
        Linear2DIndexInterpolator *line;
        if (v1x < v2x)
        {
          line = new Linear2DIndexInterpolator(v1x, v1y, v2x, v2y);
        } 
        else
        {
          line = new Linear2DIndexInterpolator(v2x, v2y, v1x, v1y);
        }

        m_globalEdgeList.insert(line);
      }
    }
  }

  // Initialize the iterator functionality
  GoToBegin();
}

template<class TImage>
PolygonScanConversionConstIterator<TImage>
::~PolygonScanConversionConstIterator()
{
  // Destructor deletes all the Linear2DIndexInterpolator's
  for (EdgeListType::const_iterator it = m_globalEdgeList.begin();
      it != m_globalEdgeList.end();
      ++it)
  {
    delete *it;
  }

  for (EdgeListType::const_iterator it = m_activeEdgeList.begin();
      it != m_activeEdgeList.end();
      ++it)
  {
    delete *it;
  }

  for (EdgeListType::const_iterator it = m_finishedEdgeList.begin();
      it != m_finishedEdgeList.end();
      ++it)
  {
    delete *it;
  }
}

template<class TImage>
void 
PolygonScanConversionConstIterator<TImage>
::GoToBegin()
{
  // Method to set the scan conversion iterator to the first index
  //
  // Moves all lines from the active and finished egde lists to the global
  // edge list. At the same time, resets the line iterators
  for (EdgeListType::const_iterator it = m_activeEdgeList.begin();
      it != m_activeEdgeList.end(); ++it)
  {
    Linear2DIndexInterpolator *line = *it;
    line->GoToBegin();
    m_globalEdgeList.insert(line);
  }
  m_activeEdgeList.clear();

  for (EdgeListType::const_iterator it = m_finishedEdgeList.begin();
      it != m_finishedEdgeList.end(); ++it)
  {
    Linear2DIndexInterpolator *line = *it;
    line->GoToBegin();
    m_globalEdgeList.insert(line);
  }
  m_finishedEdgeList.clear();

  // Without any edges there are no points inside the polygon
  if (m_globalEdgeList.empty())
  {
    m_finished = true;
    return;
  }

  // Initialize the active edge list. If this returns false, there are no
  // active edges, the polygon is empty, and we are done.
  m_finished = false;
  if (!AdvanceActiveEdges())
  {
    return;
  }

  // Now find the line where the filling starts
  if (!GoToFirstScanLineSegment())
  {
    GoToNextScanLine();
  }
}

template<class TImage>
bool 
PolygonScanConversionConstIterator<TImage>
::IsAtEnd() const
{
  return m_finished;
}

template<class TImage>
int 
PolygonScanConversionConstIterator<TImage>
::NumberOfPixelsToEndOfSegment(EdgeListType::const_iterator startEdge, int pos)
{
  // startEdge and startEdge+1 make a scan line segment from ymin to
  // ymax. NumberOfPixelsToEndOfSegment returns the number of pixels from
  // pos to ymax excluding pos.
  EdgeListType::const_iterator endEdge = startEdge;
  ++endEdge;

  int final = (*endEdge)->GetY();
  int left = final - pos;

  return (left > 0 ? left : 0);
}

template<class TImage>
bool 
PolygonScanConversionConstIterator<TImage>
::GoToFirstNonEmptyScanLineSegmentAtOrAfter(EdgeListType::const_iterator oddActiveEdge)
{
  // Method to position the iterator at the beginning of the scan line segment
  // at or below the given active edge. The segment has to contain at least
  // one pixel.
  //
  // oddActiveEdge must be just that!
  //
  // If true is returned, the iterator is at it's new position.
  // If false is returned, there is no segment to go to in the current scan
  // line and the iterator is left where it was.
  EdgeListType::const_iterator newCurrentEdge = oddActiveEdge;

  EdgeListType::const_iterator endEdge = newCurrentEdge;
  ++endEdge;

  // Search for a scan line segment that contains pixels of the polygon
  while (newCurrentEdge != m_activeEdgeList.end())
  {
    Linear2DIndexInterpolator *line = *newCurrentEdge;

    int startY = line->GetY();
    // startY is either on the polygon edge or just outside of it.
    // Find the first pixel that might be inside the polygon
    if (!line->IsYExact())
    {
      ++startY;
    }

    // Because of the increase, the segment might be empty.
    // NumberOfPixelsToEndOfSegment returns the number excluding the start
    // pixel, so we subtract 1 from startY
    if (NumberOfPixelsToEndOfSegment(newCurrentEdge, startY-1) == 0)
    {
      ++newCurrentEdge;
      ++newCurrentEdge; // Odd/Even rule for polygon filling
    }
    else
    {
      // A good segment was found! Now let's move that iterator!
      m_currentEdge = newCurrentEdge;
      m_CurrentImageIndex[m_FirstDirection] = line->GetX();
      m_CurrentImageIndex[m_SecondDirection] = startY;
      return true;
    }
  }

  // No segment could be found
  return false;
}

template<class TImage>
bool 
PolygonScanConversionConstIterator<TImage>
::GoToFirstScanLineSegment()
{
  return GoToFirstNonEmptyScanLineSegmentAtOrAfter(m_activeEdgeList.begin());
}

template<class TImage>
bool 
PolygonScanConversionConstIterator<TImage>
::GoToNextScanLineSegment()
{
  EdgeListType::const_iterator nextCurrentEdge = m_currentEdge;
  ++nextCurrentEdge;
  ++nextCurrentEdge;

  if (nextCurrentEdge == m_activeEdgeList.end())
  {
    return false;
  }
  else
  {
    return GoToFirstNonEmptyScanLineSegmentAtOrAfter(nextCurrentEdge);
  }
}

template<class TImage>
bool 
PolygonScanConversionConstIterator<TImage>
::AdvanceActiveEdges()
{
  // Edges that reached their endpoint are moved to the list of finished
  // edges. If global edges start at the new scan line, they are moved to the
  // active edges.
  //
  // False is returned if there is no new scan line. In this case, m_finished
  // is set to true as well.
  //
  // The iterator position is not changed.

  // Update the active edges and move them to a temporary storage to keep
  // them sorted after the update.
  EdgeListType temporaryActiveEdgeList;
  EdgeListType::const_iterator it = m_activeEdgeList.begin();
  while (it != m_activeEdgeList.end())
  {
    Linear2DIndexInterpolator *line = *it;

    // Remove the line from the list of active edges. If it is still active
    // after it has been updated, it will be inserted again.
    EdgeListType::const_iterator deleteIt = it;
    ++it;
    m_activeEdgeList.erase(deleteIt);

    if (line->IsAtLast())
    {
      m_finishedEdgeList.insert(line);
      continue;
    }

    // Advance active edge. If there are still global edges left, one of them
    // will start at the end of the current edge. In this case, we don't visit
    // the last pixel of the current edge.
    ++(*line);
    if (!m_globalEdgeList.empty() && line->IsAtLast())
    {
      m_finishedEdgeList.insert(line);
    }
    else
    {
      temporaryActiveEdgeList.insert(line);
    }
  }
  // copy the temporary edge list back in the active edge list
  m_activeEdgeList = temporaryActiveEdgeList;

  // If there are no more active edges, we have to insert the next remaining
  // global edge.
  if (m_activeEdgeList.empty())
  {
    // If there are no more global edges, we are done.
    if (m_globalEdgeList.empty())
    {
      m_finished = true;
      return false;
    }
    else
    {
      // Activate the next global egde
      Linear2DIndexInterpolator *line = *(m_globalEdgeList.begin());

      m_activeEdgeList.insert(line);
      m_globalEdgeList.erase(m_globalEdgeList.begin());
    }
  }

  // Now we have at least one active edge. Add global edges that start in
  // the same scan line.
  m_CurrentImageIndex[m_FirstDirection] = (*m_activeEdgeList.begin())->GetX();
  while (!m_globalEdgeList.empty())
  {
    Linear2DIndexInterpolator *line = *(m_globalEdgeList.begin());

    if (line->GetX() == m_CurrentImageIndex[m_FirstDirection])
    {
      m_activeEdgeList.insert(line);
      m_globalEdgeList.erase(m_globalEdgeList.begin());
    }
    else
    {
      // That's it, the other global edges are still too far away.
      break;
    }
  }

  return true;
}

template<class TImage>
bool 
PolygonScanConversionConstIterator<TImage>
::GoToNextScanLine()
{
  // Sets the iterator position to the first pixel inside the polygon in the
  // next scan line that contains such a pixel.
  //
  // Returns false iff no such pixel is found.
  AdvanceActiveEdges();
  if (m_finished)
  {
    return false;
  }

  while (!GoToFirstScanLineSegment())
  {
    AdvanceActiveEdges();
    if (m_finished)
    {
      return false;
    }
  }
  return true;
}

template<class TImage>
void
PolygonScanConversionConstIterator<TImage>
::operator++()
{
  // Go to the next pixel in the current scan line. If there are no more
  // pixels in this line, go to the next scan line.
  // If there are no more scan lines, we are done.
  
  // Is a pixel left in the current scan line segment?
  if (NumberOfPixelsToEndOfSegment(m_currentEdge, 
                                   m_CurrentImageIndex[m_SecondDirection]))
  {
    ++m_CurrentImageIndex[m_SecondDirection];
    return;
  }

  // Find the next scan line segment with at least one pixel
  if (GoToNextScanLineSegment())
  {
    return;
  }

  // There is no more pixel in this scan line, so we proceed to the next.
  GoToNextScanLine();
}

} // end namespace itk

#endif
