#ifndef __itkPolygonScanConversionIterator_h
#define __itkPolygonScanConversionIterator_h

#include "itkPolygonScanConversionConstIterator.h"

namespace itk
{

/**
 * \class PolygonScanConversionIterator
 * \brief Iterator that visits all points inside a complex polygon.
 *
 * PolygonScanConversionIterator is an iterator that visits all points inside
 * or on the border of a complex polygon.
 * The polygon has to be on a 2D plane orthogonal to any axis of an ND image.
 *
 * The implementation follows the active edge table approach with the
 * odd/even inclusion rule, beautifully described in
 * The Simplicity of Complex Polygons, Michael Abrash, Dr. Dobb's Journal
 * June 1991, v16, n6, p139(6)
 *
 * It's possible that the set of iterated points is not connected, for
 * example the triangle (0,0), (0,1), (3,2) looks like this:
 *  #
 *  ##
 *     #
 * This class was contributed by Benjamin King, Experimentelle
 * Radiologie, Medizinische Hochschule Hannover.
 *
 */


template<class TImage>
class PolygonScanConversionIterator : public PolygonScanConversionConstIterator<TImage>
{
public:
  /** Standard class typedefs. */
  typedef PolygonScanConversionIterator Self;

  /** Dimension of the image the iterator walks.  This constant is needed so 
   * that functions that are templated over image iterator type (as opposed to
   * being templated over pixel type and dimension) can have compile time
   * access to the dimension of the image that the iterator walks. */
  itkStaticConstMacro(ImageIteratorDimension, unsigned int,
                      TImage::ImageDimension);

  /** Define the superclass */
  typedef PolygonScanConversionConstIterator<TImage> Superclass;

  /** Inherit types from the superclass */
  typedef typename Superclass::IndexType              IndexType;
  typedef typename Superclass::IndexValueType         IndexValueType;
  typedef typename Superclass::IndexListType          IndexListType;
  typedef typename Superclass::OffsetType             OffsetType;
  typedef typename Superclass::OffsetValueType        OffsetValueType;
  typedef typename Superclass::SizeType               SizeType;
  typedef typename Superclass::SizeValueType          SizeValueType;
  typedef typename Superclass::RegionType             RegionType;
  typedef typename Superclass::SpacingType             SpacingType;
  typedef typename Superclass::PointType             PointType;
  typedef typename Superclass::ImageType              ImageType;
  typedef typename Superclass::PixelContainer         PixelContainer;
  typedef typename Superclass::PixelContainerPointer  PixelContainerPointer;
  typedef typename Superclass::InternalPixelType      InternalPixelType;
  typedef typename Superclass::PixelType              PixelType;
  typedef typename Superclass::AccessorType           AccessorType;

  /** Run-time type information (and related methods). */
  itkTypeMacro(PolygonScanConversionIterator, PolygonScanConversionConstIterator);
  
  /** Set the pixel value */
  void Set(const PixelType & value)
    {
    // Normally, this would just be the following:
    //   m_Image->SetPixel(m_CurrentImageIndex,value);
    // However, we don't want a warning about m_Image being a ConstPointer
    // in the Superclass.
    const_cast<ImageType *>(this->m_Image.GetPointer())->
      SetPixel(this->m_CurrentImageIndex,value);
    }

  /** Return a reference to the pixel 
   * This method will provide the fastest access to pixel
   * data, but it will NOT support ImageAdaptors. */
  PixelType & Value(void) 
    {
    return this->GetImage()->GetPixel(this->m_CurrentImageIndex);
    }

  /** Constructor establishes an iterator to walk along a path */
  PolygonScanConversionIterator(const ImageType *, const IndexListType &);

  /** Default Destructor. */
  virtual ~PolygonScanConversionIterator() {};
};

} // end namespace itk

#ifndef ITK_MANUAL_INSTANTIATION
#include "itkPolygonScanConversionIterator.txx"
#endif

#endif
