#include "itkPolygonScanConversionIterator.h"
#include <iostream>
#include "itkImage.h"
#include "itkImageFileWriter.h"
#include "itkRegion.h"
#include "itkTimeProbe.h"

const int Size = 180;
const int Dimension = 3;
typedef unsigned char PixelType;
typedef itk::Image<PixelType, Dimension> ImageType;
typedef ImageType::RegionType::IndexType IndexType;
typedef IndexType::IndexValueType IndexValueType;
typedef itk::PolygonScanConversionIterator<ImageType> PolygonScanConversionIteratorType;

ImageType::Pointer makeImage(const int extent)
{
  ImageType::RegionType::IndexType index;
  index.Fill(0);

  ImageType::RegionType::SizeType size;
  size.Fill(extent);

  ImageType::RegionType region;
  region.SetIndex(index);
  region.SetSize(size);

  ImageType::Pointer output = ImageType::New();
  output->SetRegions(region);
  output->Allocate();
  output->FillBuffer(0);

  return output;
}

void write(ImageType *imagePtr, char *filename)
{
  typedef itk::ImageFileWriter<ImageType> WriterType;
  WriterType::Pointer writer = WriterType::New();
  writer->SetInput(imagePtr);
  writer->SetFileName(filename);
  writer->Update();
}

void iterate(PolygonScanConversionIteratorType::IndexListType &vertices, ImageType *imagePtr)
{
  // Iterate the polygon
  PolygonScanConversionIteratorType it(imagePtr, vertices);
  it.GoToBegin();
  while (!it.IsAtEnd()) {
    ++it;
  }
}

void draw(PolygonScanConversionIteratorType::IndexListType &vertices, ImageType *imagePtr)
{
  // Draw the polygon
  PolygonScanConversionIteratorType it(imagePtr, vertices);
  it.GoToBegin();
  while (!it.IsAtEnd()) {
    it.Set(100);
    ++it;
  }
}

int main(int argc, char *argv[]) {
  PolygonScanConversionIteratorType::IndexListType vertices;
  PolygonScanConversionIteratorType::IndexType v;

  // scan convert a cube on the xy-plane, z = 8
  {
    const int cubesize = 10;
    const int zPlane = 8;
    vertices.clear();
    v[2] = zPlane;
    v[0] = 1; v[1] = 1;
    vertices.push_back(v);
    v[0] = cubesize-2; v[1] = 1;
    vertices.push_back(v);
    v[0] = cubesize-2; v[1] = cubesize-2;
    vertices.push_back(v);
    v[0] = 1; v[1] = cubesize-2;
    vertices.push_back(v);

    ImageType::Pointer output = makeImage(cubesize);
    draw(vertices, output);
    write(output, "cube.hdr");
  }

  // scan convert a diamond on the xz-plane, y = 8
  {
    const int diamondsize = 10;
    const int yPlane = 8;
    vertices.clear();
    v[1] = yPlane;
    v[0] = diamondsize/2; v[2] = 1;
    vertices.push_back(v);
    v[0] = diamondsize-1; v[2] = diamondsize/2;
    vertices.push_back(v);
    v[0] = diamondsize/2; v[2] = diamondsize-1;
    vertices.push_back(v);
    v[0] = 1; v[2] = diamondsize/2;
    vertices.push_back(v);

    ImageType::Pointer output = makeImage(diamondsize+1);
    draw(vertices, output);
    write(output, "diamond.hdr");
  }

  // scan convert a bow tie on the yz plane, x = 2
  {
    const int bowtiesize = 10;
    const int xPlane = 2;
    vertices.clear();
    v[0] = xPlane;
    v[1] = 1; v[2] = 1;
    vertices.push_back(v);
    v[1] = bowtiesize-2; v[2] = bowtiesize-2;
    vertices.push_back(v);
    v[1] = bowtiesize-2; v[2] = 1;
    vertices.push_back(v);
    v[1] = 1; v[2] = bowtiesize-2;
    vertices.push_back(v);

    ImageType::Pointer output = makeImage(bowtiesize);
    draw(vertices, output);
    write(output, "bowtie.hdr");
  }


  // scan convert a random figure on the xy plane, z = 0
  {
    const int randomsize = 100;
    const int zPlane = 0;
    const int numPoints = 50;
    vertices.clear();
    v[2] = 0;
    for (int i = 0; i < numPoints; ++i) {
      v[0] = rand() % randomsize;
      v[1] = rand() % randomsize;
      vertices.push_back(v);
    }

    ImageType::Pointer output = makeImage(randomsize);
    draw(vertices, output);
    write(output, "random.hdr");
  }

  // Rotate a star along the y axis
  {
    const int starsize = 100;
    const int corners = 7;
    const int windings = 4;
    const int radius = std::max(starsize/2-5, 1);

    ImageType::Pointer output = makeImage(starsize);
    for (int y = 0; y < starsize; ++y) {
      vertices.clear();
      v[1] = y;
      float initangle = float(y)/starsize * 2 * 3.1415926535 * windings / corners;
      for (int i = 0; i < corners; ++i) {
        float angle = initangle + float(i)/corners * windings * 2 * 3.1415926535;
        v[0] = int(round(starsize/2 + radius * cos(angle)));
        v[2] = int(round(starsize/2 + radius * sin(angle)));
        vertices.push_back(v);
      }

      draw(vertices, output);
    }
    write(output, "twistedstar.hdr");
  }

  return 0;
}
